@@ -38,8 +38,13 @@
 #include <libtracker-db/tracker-db-manager.h>
 #include <libtracker-db/tracker-db-journal.h>
 
+#include "tracker-class.h"
 #include "tracker-data-manager.h"
 #include "tracker-data-update.h"
+#include "tracker-namespace.h"
+#include "tracker-ontologies.h"
+#include "tracker-ontology.h"
+#include "tracker-property.h"
 #include "tracker-sparql-query.h"
 
 #define RDF_PREFIX TRACKER_RDF_PREFIX
@@ -75,7 +80,9 @@ load_ontology_statement (const gchar *ontology_file,
                          const gchar *predicate,
                          const gchar *object,
                          gint        *max_id,
-                         gboolean    is_new)
+                         gboolean    is_new,
+                         GHashTable *classes,
+                         GHashTable *properties)
 {
 	if (g_strcmp0 (predicate, RDF_TYPE) == 0) {
 		if (g_strcmp0 (object, RDFS_CLASS) == 0) {
@@ -97,7 +104,12 @@ load_ontology_statement (const gchar *ontology_file,
 			tracker_class_set_id (class, subject_id);
 			tracker_ontologies_add_class (class);
 			tracker_ontologies_add_id_uri_pair (subject_id, subject);
-			g_object_unref (class);
+
+			if (classes) {
+				g_hash_table_insert (classes, GINT_TO_POINTER (subject_id), class);
+			} else {
+				g_object_unref (class);
+			}
 
 		} else if (g_strcmp0 (object, RDF_PROPERTY) == 0) {
 			TrackerProperty *property;
@@ -118,7 +130,12 @@ load_ontology_statement (const gchar *ontology_file,
 			tracker_property_set_id (property, subject_id);
 			tracker_ontologies_add_property (property);
 			tracker_ontologies_add_id_uri_pair (subject_id, subject);
-			g_object_unref (property);
+
+			if (properties) {
+				g_hash_table_insert (properties, GINT_TO_POINTER (subject_id), property);
+			} else {
+				g_object_unref (property);
+			}
 
 		} else if (g_strcmp0 (object, NRL_INVERSE_FUNCTIONAL_PROPERTY) == 0) {
 			TrackerProperty *property;
@@ -390,7 +407,7 @@ load_ontology_file_from_path (const gchar        *ontology_file,
 		object = tracker_turtle_reader_get_object (reader);
 
 		load_ontology_statement (ontology_file, 0, subject, predicate, object,
-		                         max_id, is_new);
+		                         max_id, is_new, NULL, NULL);
 	}
 
 	g_object_unref (reader);
@@ -491,13 +508,22 @@ load_ontology_file (const gchar               *filename,
 	g_free (ontology_file);
 }
 
-static GHashTable *
-load_ontology_from_journal (void)
+static void
+load_ontology_from_journal (GHashTable **classes_out,
+                            GHashTable **properties_out)
 {
 	GHashTable *id_uri_map;
+	GHashTable *classes, *properties;
+
+	classes = g_hash_table_new_full (g_direct_hash, g_direct_equal,
+	                                 NULL, (GDestroyNotify) g_object_unref);
+
+	properties = g_hash_table_new_full (g_direct_hash, g_direct_equal,
+	                                    NULL, (GDestroyNotify) g_object_unref);
 
 	id_uri_map = g_hash_table_new (g_direct_hash, g_direct_equal);
 
+
 	while (tracker_db_journal_reader_next (NULL)) {
 		TrackerDBJournalEntryType type;
 
@@ -527,13 +553,15 @@ load_ontology_from_journal (void)
 			subject = g_hash_table_lookup (id_uri_map, GINT_TO_POINTER (subject_id));
 			predicate = g_hash_table_lookup (id_uri_map, GINT_TO_POINTER (predicate_id));
 
-			load_ontology_statement ("journal", subject_id, subject, predicate, object, NULL, FALSE);
+			load_ontology_statement ("journal", subject_id, subject, predicate, 
+			                         object, NULL, FALSE, classes, properties);
 		}
 	}
 
-	g_hash_table_unref (id_uri_map);
+	*classes_out = classes;
+	*properties_out = properties;
 
-	return id_uri_map;
+	g_hash_table_unref (id_uri_map);
 }
 
 
@@ -684,137 +712,6 @@ import_ontology_file (const gchar *filename,
 	}
 }
 
-static gchar *
-query_resource_by_id (gint id)
-{
-	TrackerDBCursor *cursor;
-	TrackerDBInterface *iface;
-	TrackerDBStatement *stmt;
-	gchar *uri;
-
-	g_return_val_if_fail (id > 0, NULL);
-
-	iface = tracker_db_manager_get_db_interface ();
-
-	stmt = tracker_db_interface_create_statement (iface,
-	                                              "SELECT Uri FROM Resource WHERE ID = ?");
-	tracker_db_statement_bind_int (stmt, 0, id);
-	cursor = tracker_db_statement_start_cursor (stmt, NULL);
-	g_object_unref (stmt);
-
-	tracker_db_cursor_iter_next (cursor);
-	uri = g_strdup (tracker_db_cursor_get_string (cursor, 0));
-	g_object_unref (cursor);
-
-	return uri;
-}
-
-static void
-replay_journal (void)
-{
-	GError *journal_error = NULL;
-
-	tracker_db_journal_reader_init (NULL);
-
-	while (tracker_db_journal_reader_next (&journal_error)) {
-		GError *error = NULL;
-		TrackerDBJournalEntryType type;
-		const gchar *graph, *subject, *predicate, *object;
-		gint graph_id, subject_id, predicate_id, object_id;
-
-		type = tracker_db_journal_reader_get_type ();
-		if (type == TRACKER_DB_JOURNAL_RESOURCE) {
-			TrackerDBInterface *iface;
-			TrackerDBStatement *stmt;
-			gint id;
-			const gchar *uri;
-
-			tracker_db_journal_reader_get_resource (&id, &uri);
-
-			iface = tracker_db_manager_get_db_interface ();
-
-			stmt = tracker_db_interface_create_statement (iface,
-					                              "INSERT "
-					                              "INTO Resource "
-					                              "(ID, Uri) "
-					                              "VALUES (?, ?)");
-			tracker_db_statement_bind_int (stmt, 0, id);
-			tracker_db_statement_bind_text (stmt, 1, uri);
-			tracker_db_statement_execute (stmt, &error);
-		} else if (type == TRACKER_DB_JOURNAL_START_TRANSACTION) {
-			tracker_data_begin_replay_transaction (tracker_db_journal_reader_get_time ());
-		} else if (type == TRACKER_DB_JOURNAL_END_TRANSACTION) {
-			tracker_data_commit_transaction ();
-		} else if (type == TRACKER_DB_JOURNAL_INSERT_STATEMENT) {
-			tracker_db_journal_reader_get_statement (&graph_id, &subject_id, &predicate_id, &object);
-
-			if (graph_id > 0) {
-				graph = query_resource_by_id (graph_id);
-			} else {
-				graph = NULL;
-			}
-			subject = query_resource_by_id (subject_id);
-			predicate = query_resource_by_id (predicate_id);
-
-			tracker_data_insert_statement_with_string (graph, subject, predicate, object, &error);
-		} else if (type == TRACKER_DB_JOURNAL_INSERT_STATEMENT_ID) {
-			tracker_db_journal_reader_get_statement_id (&graph_id, &subject_id, &predicate_id, &object_id);
-
-			if (graph_id > 0) {
-				graph = query_resource_by_id (graph_id);
-			} else {
-				graph = NULL;
-			}
-			subject = query_resource_by_id (subject_id);
-			predicate = query_resource_by_id (predicate_id);
-			object = query_resource_by_id (object_id);
-
-			tracker_data_insert_statement_with_uri (graph, subject, predicate, object, &error);
-		} else if (type == TRACKER_DB_JOURNAL_DELETE_STATEMENT) {
-			tracker_db_journal_reader_get_statement (&graph_id, &subject_id, &predicate_id, &object);
-
-			if (graph_id > 0) {
-				graph = query_resource_by_id (graph_id);
-			} else {
-				graph = NULL;
-			}
-			subject = query_resource_by_id (subject_id);
-			predicate = query_resource_by_id (predicate_id);
-
-			tracker_data_delete_statement (graph, subject, predicate, object, &error);
-		} else if (type == TRACKER_DB_JOURNAL_DELETE_STATEMENT_ID) {
-			tracker_db_journal_reader_get_statement_id (&graph_id, &subject_id, &predicate_id, &object_id);
-
-			if (graph_id > 0) {
-				graph = query_resource_by_id (graph_id);
-			} else {
-				graph = NULL;
-			}
-			subject = query_resource_by_id (subject_id);
-			predicate = query_resource_by_id (predicate_id);
-			object = query_resource_by_id (object_id);
-
-			tracker_data_delete_statement (graph, subject, predicate, object, &error);
-		}
-	}
-
-
-	if (journal_error) {
-		gsize size;
-
-		size = tracker_db_journal_reader_get_size_of_correct ();
-		tracker_db_journal_reader_shutdown ();
-
-		tracker_db_journal_init (NULL);
-		tracker_db_journal_truncate (size);
-		tracker_db_journal_shutdown ();
-
-		g_clear_error (&journal_error);
-	} else {
-		tracker_db_journal_reader_shutdown ();
-	}
-}
-
 static void
 class_add_super_classes_from_db (TrackerDBInterface *iface, TrackerClass *class)
 {
@@ -1552,7 +1449,8 @@ get_new_service_id (TrackerDBInterface *iface)
 gboolean
 tracker_data_manager_init (TrackerDBManagerFlags  flags,
                            const gchar           *test_schema,
-                           gboolean              *first_time)
+                           gboolean              *first_time,
+                           gboolean               journal_check)
 {
 	TrackerDBInterface *iface;
 	gboolean is_first_time_index, read_journal, check_ontology;
@@ -1572,19 +1470,24 @@ tracker_data_manager_init (TrackerDBManagerFlags  flags,
 		return TRUE;
 	}
 
+	/* Make sure we initialize all other modules we depend on */
+	tracker_ontologies_init ();
+
 	read_journal = FALSE;
 
 	if (!tracker_db_manager_init (flags, &is_first_time_index, FALSE)) {
 		return FALSE;
 	}
 
+	tracker_fts_set_map_function (tracker_ontologies_get_uri_by_id);
+
 	if (first_time != NULL) {
 		*first_time = is_first_time_index;
 	}
 
 	iface = tracker_db_manager_get_db_interface ();
 
-	if (is_first_time_index && !test_schema) {
+	if (journal_check && is_first_time_index) {
 		if (tracker_db_journal_reader_init (NULL)) {
 			if (tracker_db_journal_reader_next (NULL)) {
 				/* journal with at least one valid transaction
@@ -1608,30 +1511,37 @@ tracker_data_manager_init (TrackerDBManagerFlags  flags,
 	}
 
 	if (read_journal) {
+		GHashTable *classes = NULL, *properties = NULL;
+
 		in_journal_replay = TRUE;
 
 		/* load ontology from journal into memory */
-		load_ontology_from_journal ();
+		load_ontology_from_journal (&classes, &properties);
 
-		tracker_data_begin_replay_transaction (tracker_db_journal_reader_get_time ());
+		tracker_data_begin_db_transaction_for_replay (tracker_db_journal_reader_get_time ());
 		import_ontology_into_db (FALSE);
-		tracker_data_commit_transaction ();
+		tracker_data_commit_db_transaction ();
 
 		tracker_db_journal_reader_shutdown ();
 
-		replay_journal ();
+		tracker_data_replay_journal (classes, properties);
 
 		in_journal_replay = FALSE;
 
 		/* open journal for writing */
-		tracker_db_journal_init (NULL);
+		tracker_db_journal_init (NULL, FALSE);
 		check_ontology = TRUE;
+
+		g_hash_table_unref (classes);
+		g_hash_table_unref (properties);
 	} else if (is_first_time_index) {
 		gchar *test_schema_path = NULL;
 
 		sorted = get_ontologies (test_schema != NULL, ontologies_dir);
 
-		tracker_db_journal_init (NULL);
+		/* truncate journal as it does not even contain a single valid transaction
+		 * or is explicitly ignored (journal_check == FALSE, only for test cases) */
+		tracker_db_journal_init (NULL, TRUE);
 
 		/* load ontology from files into memory (max_id starts at zero: first-time) */
 
@@ -1648,7 +1558,7 @@ tracker_data_manager_init (TrackerDBManagerFlags  flags,
 			load_ontology_file_from_path (test_schema_path, &max_id, FALSE);
 		}
 
-		tracker_data_begin_transaction ();
+		tracker_data_begin_db_transaction ();
 		tracker_db_journal_start_transaction (time (NULL));
 
 		import_ontology_into_db (FALSE);
@@ -1662,15 +1572,15 @@ tracker_data_manager_init (TrackerDBManagerFlags  flags,
 			g_free (test_schema_path);
 		}
 
-		tracker_db_journal_commit_transaction ();
-		tracker_data_commit_transaction ();
+		tracker_db_journal_commit_db_transaction ();
+		tracker_data_commit_db_transaction ();
 
 		g_list_foreach (sorted, (GFunc) g_free, NULL);
 		g_list_free (sorted);
 		sorted = NULL;
 		check_ontology = FALSE;
 	} else {
-		tracker_db_journal_init (NULL);
+		tracker_db_journal_init (NULL, FALSE);
 
 		/* load ontology from database into memory */
 		db_get_static_data (iface);
@@ -1681,13 +1591,13 @@ tracker_data_manager_init (TrackerDBManagerFlags  flags,
 	/* ensure FTS is fully initialized */
 	tracker_db_interface_execute_query (iface, NULL, "SELECT 1 FROM fulltext.fts WHERE rowid = 0");
 
-	if (check_ontology) {
+	if (check_ontology && !test_schema) {
 		GList *to_reload = NULL;
 
 		sorted = get_ontologies (test_schema != NULL, ontologies_dir);
 
 		/* check ontology against database */
-		tracker_data_begin_transaction ();
+		tracker_data_begin_db_transaction ();
 
 		stmt = tracker_db_interface_create_statement (iface,
 		        "SELECT Resource.Uri, \"rdfs:Resource\".\"nao:lastModified\" FROM \"tracker:Ontology\""
@@ -1782,7 +1692,7 @@ tracker_data_manager_init (TrackerDBManagerFlags  flags,
 			g_list_free (to_reload);
 		}
 
-		tracker_data_commit_transaction ();
+		tracker_data_commit_db_transaction ();
 
 		g_hash_table_unref (ontos_table);
 
@@ -1803,8 +1713,10 @@ tracker_data_manager_shutdown (void)
 {
 	g_return_if_fail (initialized == TRUE);
 
+	/* Make sure we shutdown all other modules we depend on */
 	tracker_db_journal_shutdown ();
 	tracker_db_manager_shutdown ();
+	tracker_ontologies_shutdown ();
 
 	initialized = FALSE;
 }
