@@ -70,10 +70,17 @@
 
 #include <libedataserver/e-account.h>
 #include <libedataserver/e-account-list.h>
+#include <libedataserver/eds-version.h>
+
+#if EDS_CHECK_VERSION (2, 29, 1)
+#include <e-util/e-account-utils.h>
+#endif
 
 #include <libtracker-client/tracker.h>
-#include <libtracker-common/tracker-sparql-builder.h>
-#include <libtracker-common/tracker-ontology.h>
+#include <libtracker-client/tracker-sparql-builder.h>
+
+#include <libtracker-common/tracker-date-time.h>
+#include <libtracker-common/tracker-ontologies.h>
 #include <libtracker-common/tracker-utils.h>
 #include <libtracker-common/tracker-type-utils.h>
 
@@ -208,7 +215,7 @@ static ThreadPool *sparql_pool = NULL, *folder_pool = NULL;
 /* Prototype declarations */
 static void register_account (TrackerEvolutionPlugin *self, EAccount *account);
 static void unregister_account (TrackerEvolutionPlugin *self, EAccount *account);
-int e_plugin_lib_enable (EPluginLib *ep, int enable);
+int e_plugin_lib_enable (EPlugin *ep, int enable);
 static void miner_started (TrackerMiner *miner);
 static void miner_stopped (TrackerMiner *miner);
 static void miner_paused (TrackerMiner *miner);
@@ -367,6 +374,7 @@ exec_update (gpointer data, gpointer user_data)
 {
 	PoolItem *item = data;
 	GCancellable *cancel = user_data;
+	gboolean no_patience = TRUE;
 
 	if (g_cancellable_is_cancelled (cancel))
 		return;
@@ -383,14 +391,19 @@ exec_update (gpointer data, gpointer user_data)
 	}
 
 	g_mutex_lock (item->mutex);
-	if (!item->has_happened)
-		g_cond_wait (item->cond, item->mutex);
+	if (!item->has_happened) {
+		GTimeVal val;
+		g_get_current_time (&val);
+		g_time_val_add (&val, 5 * 1000000); /* 5 seconds worth of patience */
+		no_patience = g_cond_timed_wait (item->cond, item->mutex, &val);
+	}
 	g_mutex_unlock (item->mutex);
 
 	/* Don't hammer DBus too much, else Evolution's UI sometimes becomes slugish
 	 * due to a dbus_watch_handle call on its mainloop */
 
-	g_usleep (300);
+	if (no_patience)
+		g_usleep (300);
 }
 
 static gint 
@@ -1717,7 +1730,18 @@ on_register_client_qry (GPtrArray *results,
 			} else {
 				for (i = 0; i < results->len; i++) {
 					const gchar **str = g_ptr_array_index (results, i);
-					info->last_checkout = (guint64) tracker_string_to_date (str[0]);
+					GError *new_error = NULL;
+
+					info->last_checkout = (guint64) tracker_string_to_date (str[0], NULL, &new_error);
+
+					if (new_error) {
+						g_warning ("%s", new_error->message);
+						g_error_free (error);
+						g_ptr_array_foreach (results, (GFunc) g_strfreev, NULL);
+						g_ptr_array_free (results, TRUE);
+						return;
+					}
+
 					break;
 				}
 			}
@@ -2152,7 +2176,7 @@ enable_plugin (void)
 
 
 int
-e_plugin_lib_enable (EPluginLib *ep, int enabled)
+e_plugin_lib_enable (EPlugin *ep, int enabled)
 {
 	g_static_rec_mutex_lock (&glock);
 
@@ -2265,7 +2289,11 @@ tracker_evolution_plugin_init (TrackerEvolutionPlugin *plugin)
 	                         G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING,
 	                         G_TYPE_INVALID);
 
+#if EDS_CHECK_VERSION (2, 29, 1)
+	priv->accounts = g_object_ref (e_get_account_list ());
+#else
 	priv->accounts = g_object_ref (mail_config_get_accounts ());
+#endif
 
 	for (it = e_list_get_iterator (E_LIST (priv->accounts)); e_iterator_is_valid (it); e_iterator_next (it)) {
 		register_account (plugin, (EAccount *) e_iterator_get (it));
